@@ -29,6 +29,11 @@ pub fn checkout_server_derive(input: TokenStream) -> TokenStream {
 }
 
 fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
+    // panic if there are type generics defined on the Context type
+    for _ in ast.generics.type_params() {
+        panic!("can't have generic types defined on Context type")
+    }
+
     let context_struct_name = &ast.ident;
 
     let server_struct_attribute = ast
@@ -42,6 +47,26 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
         parse2(server_struct_attribute.tokens.clone()).expect("invalid struct_name");
     let server_struct_name = &server_struct.name;
 
+    let mut impl_generics_container = ast.generics.clone();
+    impl_generics_container.params.clear();
+    impl_generics_container
+        .params
+        .push(syn::parse_quote!('common));
+
+    let mut ty_generics_container = ast.generics.clone();
+    ty_generics_container.params.clear();
+
+    for _ in ast.generics.lifetimes() {
+        ty_generics_container
+            .params
+            .push(syn::parse_quote!('common));
+    }
+
+    let (impl_generics, _, _) = impl_generics_container.split_for_impl();
+    let (_, ty_generics, _) = ty_generics_container.split_for_impl();
+    let common_lifetime_def = syn::LifetimeDef::new(syn::parse_quote!('common));
+    let common_lifetime = common_lifetime_def.lifetime;
+
     let gen = quote! {
         mod checkout_server {
             pub use checkout_core::dropshot::endpoint;
@@ -102,7 +127,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             }
         }
 
-        impl checkout_server::Server<#context_struct_name> for #server_struct_name {
+        impl #impl_generics checkout_server::Server<#common_lifetime, #context_struct_name #ty_generics> for #server_struct_name {
             type Context = checkout_server::ServerContext;
 
             fn api_description() -> checkout_server::ApiDescription<checkout_server::ServerContext> {
